<?php

class cfgmgr_NginxVhostConf extends cfgmgr_Conf {

  public $service = 'nginx';

  protected $cfg_registry = array(
    // Port
    'port' => array(
      '#type' => 'ask',
      '#question' => "What is the port your vhost should listen on?",
      '#default' => 80,
    ),
    'ssl_port' => array( 
      '#type' => 'ask',
      '#question' => "What is the SSL port your vhost should listen on?",
      '#default' => 443,
    ), 
    // SSL settings
    'ssl_support' => array(
      '#type' => 'confirm',
      '#question' => "Do you want to support SSL",
      '#default' => FALSE,
    ),
    // IP
    'ssl_ip' => array(
      '#type' => 'ask',
      '#question' => "What is the SSL IP your vhost should listen on?",
    ),
    // Path to SSL cert
    'ssl_cert' => array(
      '#type' => 'ask',
      '#question' => "Where is the SSL cert located on the server",
      '#default' => '/etc/ssl/certs/!ns-cert.pem',
    ),
    // Path to SSL cert
    'ssl_cert_key' => array(
      '#type' => 'ask',
      '#question' => "Where is the SSL cert key located on the server",
      '#default' => '/etc/ssl/certs/!ns.key',
    ),
    // Server name details
    'servername' => array(
      '#type' => 'ask',
      '#question' => "What is the servername for this vhost",
    ),
    'serveraliases' => array(
      '#type' => 'ask',
      '#question' => "Do you want to support any server aliases? If so name each one seperated by a space",
    ),
    'serverredirects' => array(
      '#type' => 'ask',
      '#question' => "Name any domains that should redirect to the servername seperated by spaces",
    ),
    // Upstream Reference
    'upstream' => array(
      '#type' => 'ask',
      '#question' => "What will the Upstream name be that Nginx will forward Drupal requests onto?",
      '#default' => '!ns',
    ),
    'upstream_type' => array(
      '#type' => 'select',
      '#question' => 'Which upstream method is being used?',
      '#options' => array(
        array('!key' => 'fcgi', '!value' => 'PHP FastCGI (FPM)'),
        array('!key' => 'apache', '!value' => 'Proxy to Apache')
      ),
      '#default' => 'fcgi',
    ),
  );

  /**
   * Generate a vhost configuration file.
   */
  public function conf() {
    $vhost = file_get_contents(dirname(__FILE__) . '/example.com.conf');

    if (strpos($this->servername, 'www') !== 0) {
      $this->cfg_registry['serverredirects']['#default'] = 'www.' . $this->servername;
      $this->cfg_handler->registry_variables($this->cfg_registry);
    }
    drush_set_option($this->service . '-error_log', '/var/log/sitelogs/!ns/nginx_error.log'); 
    drush_set_option($this->service . '-access_log', '/var/log/sitelogs/!ns/nginx_access.log'); 

    $replacements = array(
      'example.com' => $this->servername, 
      '!ns' => $this->cfg_handler->ns,
      'server_name example.com' => 'server_name ' . $this->servername . ' ' . $this->serveraliases,
      '!redirects' => $this->serverredirects,
      '[::]:80' => '[::]:' . $this->port, 
      'access_log  /var/log/sitelogs/!ns/nginx_access.log' => 'access_log ' . drush_get_option($this->service . '-access_log'),
      'error_log  /var/log/sitelogs/!ns/nginx_error.log' => 'error_log ' . drush_get_option($this->service . '-error_log'),
    );

    if ($this->ssl_support) {
      $vhost .= file_get_contents(dirname(__FILE__) . '/ssl.example.com.conf');
      $replacements['[::]:443 ssl'] = '[::]:' . $this->ssl_port . ' ssl';
      $replacements['!ssl_ip'] = $this->ssl_ip;
      $replacements['/etc/ssl/certs/example-cert.pem'] = $this->ssl_cert;
      $replacements['/etc/ssl/private/example.key'] = $this->ssl_cert_key;
    }

    // Conf files are configured by default to use CGI
    // so if the setup is to proxy to apache, change things up.
    $upstream = $this->upstream;
    if ($this->upstream_type == 'apache') {
      $replacements['fastcgi_pass !upstream'] = '#fastcgi_pass ' . $upstream;
      $replacements['#proxy_pass http://!upstream'] = 'proxy_pass http://' . $upstream;
    }
    $replacements['!upstream'] = $upstream;

    // Automate how the vhost is constructed. This can 
    // only be done if Drupal is bootstrapped.
    if (function_exists('module_exists')) {
      if ($purl_enabled = module_exists('purl')) {
        // Switch configuration around to use spaces conf.
        $replacements['include includes/drupal.conf;'] = '#include includes/drupal.conf;';
        $replacements['#include includes/drupal_spaces.conf;'] = 'include includes/drupal_spaces.conf;';
        if (module_exists('boost')) {
          $replacements['#include includes/drupal_spaces_boost.conf;'] = 'include includes/drupal_spaces_boost.conf;';
        }
      }
      if (!$purl_enabled && module_exists('boost')) {
        $replacements['#include includes/drupal_boost.conf;'] = 'include includes/drupal_boost.conf;';
      }
    }
    return strtr($vhost, $replacements);
  }
}
