<?php

/**
 * Base handler to deal with configuration of services.
 */
class cfg_handler_drushInteractive{

  /**
   * Namespace like drupal-site-[project].
   */
  protected $ns = '';


  /**
   * Service name.
   */
  protected $service = '';

  /**
   * A service's variable registry.
   */
  protected $registry = array();

  public function __construct($ns = '') {
      $this->ns = $ns;
  }

  /**
   * Magic method: Retrieve a services variable value.
   */
  public function __get($key) {
    if ($key == 'ns') {
      return $this->ns;
    }
    if (!isset($this->registry[$key])) {
      return drush_set_error("[$this->service]: No such key: $key");
    }
    if (drush_get_option($this->service . '-' . $key, NULL) !== NULL) {
      return drush_get_option($this->service . '-' . $key, NULL);
    }
    $var = $this->registry[$key];
    $var['#message'] = "[$this->service] " . $var['#question'];
    switch ($var['#type']) {
      case 'ask':
        if (!isset($var['#default'])) {
          $var['#default'] = 'none';
          $var['#no_default'] = TRUE;
        }
        $value = drush_prompt($var['#message'], $var['#default']);
        if (isset($var['#no_default']) && $value == 'none') {
          $value = '';
        }
      break;
      case 'confirm':
        $value = drush_confirm($var['#message']);
      break;
      case 'select':
        $value = drush_choice($var['#options'], $var['#message'], $var['#default']);
      break;
      case 'notify':
        drush_print($var['#message']);
        sleep(2);
        return;
      break;
      default:
        return drush_set_error("[$this->service] Unknown variable method: {$var['#type']}");
      break;
    }
    $value = str_replace('!ns', $this->ns, $value);
    drush_set_option($this->service . '-' . $key, $value);
    return $value;
  }

  /**
   * Register a services variables.
   */
  public function registry_variables($variables = FALSE) {
    if ($variables) {
      $this->registry = $variables; 
      foreach ($this->registry as $key => &$info) {
        if (isset($info['#default'])) {
          $info['#default'] = str_replace('!ns', $this->ns, $info['#default']);
        }
      }
    }
    return $this->registry;
  }

  public function setService($service) {
    $this->service = $service;
  }

  /**
   * Reset stored variables from a namespace.
   *
   * As Drush is stateless, just return true.
   */
  public function reset_variables($ns, $service) {
    return TRUE;
  }
}
